--[[
Name: GPSLib-1.0
Revision: $Rev: 35984 $
Author(s): Cryect (cryect@gmail.com)
Website: http://www.wowace.com/
Documentation: http://www.wowace.com/wiki/GPSLib
SVN: http://svn.wowace.com/root/trunk/GPSLib/
Description: A library for getting real world coordinates both in dungeons/outside

WARNING - Current version lacks validation/calibration code as well doesn't have the proper equation in for Outlands and new Starting Areas

Coordinates returned at the moment have a different scalar than game file map coordinates but share the same origin

Player Coordinates are often achieved via asking for map coordinates though this runs into a few issues.

1. No way to get coordinates inside of dungeons (if there is no dungeon map how can they give you coordinates).
2. No easy transformation to real map coordinates (the actual coordinates used for positioning objects in the actual gamefiles, though there is actual 2 
coordinate systems used in the gamefiles but thats another issue)

The solution for this is to use minimap ping positions. When the game is first loaded the last ping position is initialized to 0, 0 in map coordinates. By 
tracking the delta every time a new ping occurs its possible to then calculate current real map coordinates.

Account Var - Store Last Known Ping Coords
Char Var - Store Last Known Pos

PseudoCode for Calibration on Loading

PingDelta=0
GPSState=Valid --Start off assuming the state will be correct, marked invalid if we can't verify the data is probably correct
If Outside Then
	Calibrate Current Ping Delta Based on Current Map Coord
else
	if PlayerLastPos then
		--Is the current Ping Delta of 0 Valid?
		if PlayerLastPos ~= PingPos then
			--Lets instead try the last Ping Delta stored in Global Variables then
			--This happens when you logout from one character to another and a ping has occured so 0 isn't correct
			PingDelta=LastKnownPingDelta
			if PlayerLastPos ~= PingPos then
				--Didn't match up so likely a crash occured meaning variables didn't get saved
				PingDelta=0
				GPSState=Invalid --Unsure if this is actually right or not
				
			end
		end
	else
		PingDelta=0
		GPSState=Invalid --Unsure if this is actually right or not
	end
end
	
If the state ends up being invalid it will then recalibrate when the player goes outside. Ideally the addon will later be able to calibrate

The code in this library is actually based on my old IMBA Tracker code but was never used for much so little was known of it so figure I put it in a form others can use
]]

local major, minor = "GPS-1.0", "$Revision: 35984 $"

if not AceLibrary then error(major .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(major, minor) then return end

--The various data that is stored
local GPSData = AceLibrary("AceAddon-2.0"):new("AceDB-2.0")
GPSData:RegisterDB("GPSDB","GPSPerCharDB")

--GPS_Minimap_Zoom stores the various zoom multipliers for the minimap
local GPS_Minimap_Zoom={};
GPS_Minimap_Zoom[0]={}
GPS_Minimap_Zoom[1]={}
--For Outside of WMO's
GPS_Minimap_Zoom[0][0]=1;
GPS_Minimap_Zoom[0][1]=6/7;
GPS_Minimap_Zoom[0][2]=5/7;
GPS_Minimap_Zoom[0][3]=4/7;
GPS_Minimap_Zoom[0][4]=3/7;
GPS_Minimap_Zoom[0][5]=2/7;

--For Inside of WMO's
GPS_Minimap_Zoom[1][0]=1*0.643;
GPS_Minimap_Zoom[1][1]=4/5*0.643;
GPS_Minimap_Zoom[1][2]=3/5*0.643;
GPS_Minimap_Zoom[1][3]=2/5*0.643;
GPS_Minimap_Zoom[1][4]=4/15*0.643;
GPS_Minimap_Zoom[1][5]=1/6*0.643;

--The various equations to convert map coordinates to Ping coordinates
--Linear equations fitted via linear least squares of a decent sampling of numbers ~25
--Introduces error on the order of +/-0.005 of ping coordinates
local GPS_MapEquations={}
GPS_MapEquations[1]={mx=-78.85670491;bx=36.57127371;my=52.57102443;by=-27.42828408}	--Eastern Kingsdom
GPS_MapEquations[2]={mx=-75.42874908;bx=34.2859094;my=50.2865016;by=-16.00008308}	--Kalimdor
GPS_MapEquations[3]={mx=-37.42301979;bx=27.84865433;my=24.94868364;by=-12.47434192}	--Outlands


--Few local variables reused so they don't keep getting GC'ed
local x,y, zoom, equ


--Event Handling
local GPSEvents = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0")

function GPSEvents:OnEnable()
	self:RegisterEvent("MINIMAP_PING")
	self:ScheduleRepeatingEvent(GPSEvents.UpdatePlayerPos,0)

	GPSEvents:Validate()
end

local GPS_PingDelta={x=0;y=0}
local GPS_LastPos={x=0;y=0}
local GPS_Loaded=false

--Every ping we need to calculate the new ping delta
function GPSEvents:MINIMAP_PING()
	x,y=Minimap:GetPingPosition();
	zoom = GPS_Minimap_Zoom[GPS_isMinimapInsideWMO()][Minimap:GetZoom()];
	GPS_PingDelta.x=GPS_PingDelta.x+GPS_LastPos.x-x*zoom;
	GPS_PingDelta.y=GPS_PingDelta.y+GPS_LastPos.y-y*zoom;

	GPS_LastPos.x=x;
	GPS_LastPos.y=y;
		
	GPSData.db.account.PingDelta=GPS_PingDelta
end

function GPSEvents:Validate()
	--DEFAULT_CHAT_FRAME:AddMessage("Loaded add Validation Code")
	GPS_Loaded=true
	if type(GPSData.db.char.LastPlayerPos)~="table" then
		GPSData.db.char.LastPlayerPos={}
	end
end

function GPSEvents:UpdatePlayerPos()
	x,y = Minimap:GetPingPosition();
	zoom = GPS_Minimap_Zoom[GPS_isMinimapInsideWMO()][Minimap:GetZoom()];

	GPS_LastPos.x=x*zoom
	GPS_LastPos.y=y*zoom	


	if GPSLib_Loaded then
		x=x*zoom+GPS_PingDelta.x;
		y=y*zoom+GPS_PingDelta.y;
	
		GPSData.db.char.LastPlayerPos.x=x
		GPSData.db.char.LastPlayerPos.y=y
	end
end








local tempzoom, InsideWMO

--Function from gatherer though slightly changed to be more correctly named
function GPS_isMinimapInsideWMO()
	tempzoom = 0;
	InsideWMO = 0;

	if (GetCVar("minimapZoom") == GetCVar("minimapInsideZoom")) then
		if (GetCVar("minimapInsideZoom")+0 >= 3) then 
			Minimap:SetZoom(Minimap:GetZoom() - 1);
			tempzoom = 1;
		else
			Minimap:SetZoom(Minimap:GetZoom() + 1);
			tempzoom = -1;
		end
	end
	if (GetCVar("minimapInsideZoom")+0 == Minimap:GetZoom()) then InsideWMO = 1; end
	Minimap:SetZoom(Minimap:GetZoom() + tempzoom);
	return InsideWMO;
end








--Define the library functions
--Test Code 
--/script GPS=AceLibrary("GPS-1.0");x,y=GPS:GetPosition();mx,my=GPS:GetPosition_ViaMap();DEFAULT_CHAT_FRAME:AddMessage("Pos: "..x..", "..y.."   Map Pos: "..mx..", "..my)
--/script GPS=AceLibrary("GPS-1.0");GPS:SetCoordinateSystem(GPS_COORDS_ADT_1);x,y=GPS:GetPosition();mx,my=GPS:GetPosition_ViaMap();DEFAULT_CHAT_FRAME:AddMessage("Pos: "..x..", "..y.." Map Pos: "..mx..", "..my)
local lib={}

--The various coordinate systems
GPS_COORDS_MINIMAP	= 0
GPS_COORDS_MAP		= 1
GPS_COORDS_ADT_1	= 2
GPS_COORDS_ADT_2	= 3



--This gets player position via the Minimap Ping System
function lib:GetPosition()
	x,y = Minimap:GetPingPosition();
	zoom = GPS_Minimap_Zoom[GPS_isMinimapInsideWMO()][Minimap:GetZoom()];

	x=x*zoom+GPS_PingDelta.x;
	y=y*zoom+GPS_PingDelta.y;
	
	if self.CoordSystem==GPS_COORDS_ADT_1 then
		x=-466.666666666*x+17066.66666666666
		y=466.666666666*y+17066.66666666666
	elseif self.CoordSystem==GPS_COORDS_ADT_2 then
		x=466.6666666666*x
		y=466.6666666666*y
	elseif self.CoordSystem==GPS_COORDS_MAP then
		SetMapToCurrentZone()
		local continent=GetCurrentMapContinent();
		SetMapZoom(continent)
		equ=GPS_MapEquations[continent]
		if equ then
			x=(x-equ.bx)/equ.mx
			y=(y-equ.by)/equ.my
		end
	end

	return x, y
end

--Uses the Map Coordinate System to retrieve coordinates
function lib:GetPosition_ViaMap()
	SetMapToCurrentZone()
	local continent=GetCurrentMapContinent();
	SetMapZoom(continent)
	x, y = GetPlayerMapPosition("player");

	if self.CoordSystem~=GPS_COORDS_MAP then
		local equ=GPS_MapEquations[continent]
		if not equ then
			equ={mx=1;bx=0;my=1;by=0}
		end
		x=x*equ.mx+equ.bx
		y=y*equ.my+equ.by

		if self.CoordSystem==GPS_COORDS_ADT_1 then
			x=-466.666666666*x+17066.66666666666
			y=466.666666666*y+17066.66666666666
		elseif self.CoordSystem==GPS_COORDS_ADT_2 then
			x=466.6666666666*x
			y=466.6666666666*y
		end
	end

	return x, y
end

function lib:SetCoordinateSystem(System)
	self.CoordSystem=System
end

AceLibrary:Register(lib, major, minor)