local major, minor = "Mode7-1.0", "$Revision: 36430 $"

if not AceLibrary then error(major .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(major, minor) then return end

local lib={}
local Mode7={}

function lib:CreateTexture(parent, name, layer, relative,relativeTo,offsetX,offsetY,Width,Height, mode7func)
	local f=CreateFrame("FRAME",name,parent)

	f:SetPoint(relative,parent,relativeTo,offsetX,offsetY)
	f:SetWidth(Width)
	f:SetHeight(Height)

	f.Textures={}

	local CurY,NexY,Inc

	Inc=1/Height
	CurY=0
	NexY=Inc
	
	f.Mode7Func=mode7func or DefaultScaler

	for i=1, Height do
		local t=f:CreateTexture(nil,layer)
		t:SetPoint("TOPLEFT",f,"TOPLEFT",0,-(i-1))
		t:SetWidth(Width)
		t:SetHeight(1)
		t.id=i
		
		t.Mode7Precalc=i/Height
		t.Mode7Scale=f.Mode7Func(t.Mode7Precalc)
		t.UpperY=CurY
		t.LowerY=NexY
		t.CalcTexCoords=Mode7.CalcTexCoords	
		
		table.insert(f.Textures,t)
		
		CurY=NexY
		NexY=NexY+Inc
	end

	f.SetTexture=Mode7.SetTexture
	f.SetCoords=Mode7.SetCoords
	f.SetColor=Mode7.SetColor
	f.AdjustScaling=Mode7.AdjustScaling

	return f
end

AceLibrary:Register(lib, major, minor)


function DefaultScaler(a)
	return a
end

function Mode7:SetTexture(name)
	for _, t in pairs(self.Textures) do
		t:SetTexture(name)
	end
end

function Mode7:SetCoords(scale,xoffset,yoffset,angle)
	if not self:IsVisible() then
		return
	end

	for _, t in pairs(self.Textures) do
		t:CalcTexCoords(scale,xoffset,yoffset,angle)
	end
end

function Mode7:SetColor(r,g,b,a)
	for _, t in pairs(self.Textures) do
		t:SetVertexColor(r,g,b,a)
	end
end

function Mode7:AdjustScaling()
	for _, t in pairs(self.Textures) do
		t.Mode7Scale=self.Mode7Func(t.Mode7Precalc)
	end
end


function setCoords(t, A, B, C, D, E, F)
	local det = A*E - B*D;
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy;

	local Y1,Y2
	Y1=t.UpperY
	Y2=t.LowerY
	
	ULx, ULy = ( -Y1*B + B*F - C*E ) / det, ( Y1*A - A*F + C*D ) / det;
	LLx, LLy = ( -Y2*B + B*F - C*E ) / det, ( Y2*A - A*F + C*D ) / det;
	URx, URy = ( E - Y1*B + B*F - C*E ) / det, ( -D + Y1*A -(A*F) + C*D ) / det;
	LRx, LRy = ( E - Y2*B + B*F - C*E ) / det, ( -D + Y2*A -(A*F) + C*D ) / det;

	t:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy);
end

local CosA,SinA
local Mode7_A, Mode7_B, Mode7_C, Mode7_D, Mode7_E, Mode7_F

function Mode7:CalcTexCoords(scale,xoffset,yoffset,angle)
	local Radian=math.pi*angle/180
	
	xoffset=(xoffset-0.5)
	yoffset=(yoffset-0.5)
	--Only do this stuff once each time
	if self.id==1 then
		CosA=math.cos(Radian)
		SinA=math.sin(Radian)		
	end

	--Could precalc parts of this but not really worth it due the Mode7Scale var changes with lines
	Mode7_A=self.Mode7Scale*scale*CosA
	Mode7_B=self.Mode7Scale*scale*SinA
		
	Mode7_D=self.Mode7Scale*-scale*SinA
	Mode7_E=self.Mode7Scale*scale*CosA

	Mode7_C=self.Mode7Scale*scale*(xoffset*CosA+yoffset*SinA)+0.5
	Mode7_F=self.Mode7Scale*scale*(-xoffset*SinA+yoffset*CosA)+0.5
	

	setCoords(self,Mode7_A,Mode7_B,Mode7_C,Mode7_D,Mode7_E,Mode7_F)

	--self:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy)
end